package models

import (
	"encoding/base64"

	"github.com/jackc/pgtype"
)

// TemplateDB is a Template stored in db
// dbtable:"template" dbpkey:"id"
type TemplateDB struct {
	Template

	ID uint64 `db:"id" json:"-"`

	InternalVersion uint64       `db:"internal_version" json:"-"`
	BytesBody       pgtype.Bytea `db:"body"             json:"-"`
}

// FromDB loads the db fields into the api facing fields.
func (t *TemplateDB) FromDB() {
	t.BodyFormat = BodyFormatText
	t.Body = string(t.BytesBody.Bytes)
}

// ToDB copies the api facing files into the db fields.
func (t *TemplateDB) ToDB() error {
	b, err := t.Bytes()
	if err != nil {
		return err
	}

	return t.BytesBody.Set(b)
}

// ForceBase64 makes sure the body is base64 encoded.
func (t *TemplateDB) ForceBase64() {
	if t.BodyFormat == BodyFormatText {
		t.Body = base64.StdEncoding.EncodeToString([]byte(t.Body))
		t.BodyFormat = BodyFormatB64
	}
}
