package handlers

import (
	"context"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/user_management"
)

// NewUserListHandler returns a UserListHandler.
func NewUserListHandler(db *sqlx.DB) *UserListHandler {
	return &UserListHandler{
		db: db,
	}
}

// UserListHandler handles GET /user.
type UserListHandler struct {
	db *sqlx.DB
}

func (h *UserListHandler) handle(
	ctx context.Context,
) ([]*models.Userinfo, error) {
	var (
		log  = zerolog.Ctx(ctx)
		list []*models.Userinfo
	)
	if err := database.SelectContext(ctx, h.db, &list,
		squirrel.Select(models.UserinfoColumns...).
			From(models.UserDBTable).
			OrderBy(models.UserDBCreatedAtColumn),
		log,
	); err != nil {
		return nil, err
	}

	for _, user := range list {
		if err := HandleUserActivity(ctx, h.db, user, log); err != nil {
			return nil, err
		}
	}

	return list, nil
}

// Handle handles the incoming request.
func (h *UserListHandler) Handle(
	params user_management.UserListParams, principal *models.Principal,
) middleware.Responder {
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	if !principal.IsAdmin {
		log.Error().Msg("only admins can list all the users")

		return user_management.NewUserListUnauthorized()
	}
	list, err := h.handle(params.HTTPRequest.Context())
	if err != nil {
		return InternalError(err, log, false)
	}

	return user_management.NewUserListOK().WithPayload(list)
}
